/*
 * sample_observer.c
 *
 * Example to illustrate the use of the observer components.
 *
 * Copyright © 2010 Salomon Sickert
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 3 of the licence or (at
 * your option) any later version.
 *
 * See the included COPYING file for more information.
 *
 * Authors:
 *    Salomon Sickert <sickert@in.tum.de>
 */

#include <glib.h>
#include "../libtaskview/taskview.h"
//#include <libtaskview/taskview.h>

// Create monitor
//--> listen
// Create app
//--> listen
// Create proxies
//--> listen

static void
on_property_changed (GObject *caller, GParamSpec *arg1, gpointer user_data)
{
  g_debug ("Property changed: %s", arg1->name);
}

static void
on_view_appeared (TaskviewApplicationProxy *caller, const gchar *object_path, gpointer user_data)
{
  TaskviewGenericProxy *generic;
  TaskviewIOProxy *io;

  generic = taskview_application_proxy_get_generic_proxy (caller, object_path);
  io = taskview_application_proxy_get_io_proxy (caller, object_path);

  if (io != NULL)
    g_signal_connect (io, "notify", G_CALLBACK (on_property_changed), user_data);
  else 
    g_signal_connect (generic, "notify", G_CALLBACK (on_property_changed), user_data);

  g_debug ("View (%s) appeared: %s", io != NULL ? "IO" : "Generic", object_path);
}

static void
on_view_disappeared (TaskviewApplicationProxy *caller, const gchar *object_path, gpointer user_data)
{
  g_debug ("View disappeared: %s", object_path);
}

static void
on_worker_appeared (TaskviewMonitor *caller, const gchar *dbus_name, gpointer user_data)
{
  TaskviewApplicationProxy * app = taskview_monitor_get_app_proxy (user_data, dbus_name);

  g_signal_connect (app, "taskview-added", G_CALLBACK (on_view_appeared), user_data);
  g_signal_connect (app, "taskview-removed", G_CALLBACK (on_view_disappeared), user_data);

  g_debug ("Worker appeared: %s", dbus_name);
}

static void
on_worker_disappeared (TaskviewMonitor *caller, const gchar *dbus_name, gpointer user_data)
{
  g_debug ("Worker disappeared: %s", dbus_name);
}

int
main (int argc, char *argv[])
{
  GMainLoop *loop;
  TaskviewMonitor *monitor;

  g_type_init ();

  monitor = g_object_new (TASKVIEW_TYPE_MONITOR, NULL);

  g_signal_connect (monitor, "worker-appeared", G_CALLBACK (on_worker_appeared), monitor);
  g_signal_connect (monitor, "worker-disappeared", G_CALLBACK (on_worker_disappeared), NULL);

  loop = g_main_loop_new (NULL, FALSE);
  g_main_loop_run (loop);
}
